package djmil.cordacheckers.contracts;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.Command;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameBoardContract implements net.corda.v5.ledger.utxo.Contract {

    private final static Logger log = LoggerFactory.getLogger(GameBoardContract.class);

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameBoardContract.verify()");

        final List<Command> commandList = trx.getCommands();
        requireThat(commandList.size() == 1, REQUIRE_SINGLE_COMMAND);

        final Command command = commandList.get(0);

        if (command instanceof GameProposalCommand) {
            log.info("GameBoardContract.verify() as GameProposalCommand "+(GameProposalCommand)command);
        } else 
        if (command instanceof GameBoardCommand) {
            log.info("GameBoardContract.verify() as GameBoardCommand "+((GameBoardCommand)command).getType());
            switch (((GameBoardCommand)command).getType()) {
                case SURRENDER:
                break;
                case DRAW:
                break;
                case VICTORY:
                break;
                case MOVE:
                break;
            }
        }
    }

    @Suspendable
    public static GameProposalState getReferanceGameProposalState(UtxoLedgerTransaction trx) {
        final List<Command> commandList = trx.getCommands();
        requireThat(commandList.size() == 1, REQUIRE_SINGLE_COMMAND);

        final Command command = commandList.get(0);
        if (command instanceof GameProposalCommand && (GameProposalCommand)command == GameProposalCommand.ACCEPT) {
            return trx.getInputStates(GameProposalState.class)
                .stream()
                .reduce( (a, b) -> {throw new IllegalStateException(SINGLE_STATE_EXPECTED);} )
                .get();
        } else 
        if (command instanceof GameBoardCommand) {
            return trx.getReferenceStates(GameProposalState.class)
                .stream()
                .reduce( (a, b) -> {throw new IllegalStateException(SINGLE_STATE_EXPECTED);} )
                .get();
        }

        throw new IllegalStateException(NO_REFERANCE_GAMEPROPOSAL_STATE_FOR_TRXID +trx.getId());
    }
    
    private static void requireThat(boolean asserted, String errorMessage) {
        if(!asserted) {
            throw new CordaRuntimeException("Failed requirement: " + errorMessage);
        }
    }

    static final String REQUIRE_SINGLE_COMMAND = "Require a single command";
    static final String UNKNOWN_COMMAND = "Unsupported command";

    static final String SINGLE_STATE_EXPECTED = "Single state expected";
    static final String NO_REFERANCE_GAMEPROPOSAL_STATE_FOR_TRXID = "No reference GamePropsal state found for trx.id ";
}
