package djmil.cordacheckers.contracts;

import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.Command;
import net.corda.v5.ledger.utxo.StateAndRef;

public enum GameProposalCommand implements Command { 
    CREATE,
    ACCEPT,
    REJECT,
    CANCEL;

    public MemberX500Name getInitiator(GameProposalState gameProposalState) {
        switch (this) {
            case CREATE:
            case CANCEL:
                return gameProposalState.getIssuer();
            case ACCEPT:
            case REJECT:
                return gameProposalState.getAcquier();
            default:
                throw new RuntimeException(UNSUPPORTED_VALUE_OF + this.name());
        }
    }

    public MemberX500Name getRespondent(GameProposalState gameProposalState) {
        switch (this) {
            case CREATE:
            case CANCEL:
                return gameProposalState.getAcquier();
            case ACCEPT:
            case REJECT:
                return gameProposalState.getIssuer();
            default:
                throw new RuntimeException(UNSUPPORTED_VALUE_OF + this.name());
        }
    }

    public MemberX500Name getInitiator(StateAndRef<GameProposalState> utxoGameProposal) {
        return getInitiator(utxoGameProposal.getState().getContractState());
    }

    public MemberX500Name getRespondent(StateAndRef<GameProposalState> utxoGameProposal) {
        return getRespondent(utxoGameProposal.getState().getContractState());
    }

    public static final String UNSUPPORTED_VALUE_OF = "Unsupported GameProposalCommand value: ";
}
