package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameBoardContract;
import djmil.cordacheckers.states.Piece.Color;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.ledger.utxo.BelongsToContract;
import net.corda.v5.ledger.utxo.ContractState;
import net.corda.v5.ledger.utxo.StateAndRef;

@BelongsToContract(GameBoardContract.class)
public class GameBoardState implements ContractState {

    Map<Integer, Piece> board;
    Piece.Color moveColor;
    String message;
    UUID id;
    List<PublicKey> participants;

    public GameBoardState(
        StateAndRef<GameProposalState> utxoGameBoard
    ) {
        this.board = new LinkedHashMap<Integer, Piece>(initialBoard);
        this.moveColor = Piece.Color.WHITE;
        this.message = null;
        this.id = UUID.randomUUID();
        this.participants = utxoGameBoard.getState().getContractState().getParticipants();
    }
 
    @ConstructorForDeserialization
    public GameBoardState(Map<Integer, Piece> board, Color moveColor, String message, UUID id,
            List<PublicKey> participants) {
        this.board = board;
        this.moveColor = moveColor;
        this.message = message;
        this.id = id;
        this.participants = participants;
    }

    public Map<Integer, Piece> getBoard() {
        return board;
    }

    public Piece.Color getMoveColor() {
        return moveColor;
    }

    public String getMessage() {
        return message;
    }

    public UUID getId() {
        return id;
    }

    public List<PublicKey> getParticipants() {
        return participants;
    }

    public final static Map<Integer, Piece> initialBoard =  Map.ofEntries(
        // Inspired by Checkers notation rules: https://www.bobnewell.net/nucleus/checkers.php
        Map.entry( 1, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 2, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 3, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 4, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 5, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 6, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 7, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 8, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry( 9, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry(10, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry(11, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),
        Map.entry(12, new Piece(Piece.Color.BLACK, Piece.Type.MAN)),

        Map.entry(21, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(22, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(23, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(24, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(25, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(26, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(27, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(28, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(29, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(30, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(31, new Piece(Piece.Color.WHITE, Piece.Type.MAN)),
        Map.entry(32, new Piece(Piece.Color.WHITE, Piece.Type.MAN))
        );
}
