package djmil.cordacheckers.gameboard;

import java.util.UUID;

import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.Piece;
import net.corda.v5.base.types.MemberX500Name;

// Class to hold results of the List flow.
// JsonMarshallingService can only serialize simple classes that the underlying Jackson serializer recognises, 
// hence creating a DTO style object which consists only of Strings and a UUIDs. It is possible to create custom
// serializers for the JsonMarshallingService in the future.

public class ListItem {
    public final String opponentName;
    public final Piece.Color opponentColor;
    public final Boolean opponentMove;
    public final Object board;
    public final String message;
    public final UUID id;

    // Serialisation service requires a default constructor
    public ListItem() {
        this.opponentName = null;
        this.opponentColor = null;
        this.opponentMove = null;
        this.board = null;
        this.message = null;
        this.id = null;
    }

    public ListItem(GameBoardState gameBoard, MemberX500Name opponentName, Piece.Color opponentColor) {
        this.opponentName  = opponentName.getCommonName();
        this.opponentColor = opponentColor;
        this.opponentMove  = opponentColor == gameBoard.getMoveColor();
        this.board   = gameBoard.getBoard();
        this.message = gameBoard.getMessage();
        this.id      = gameBoard.getId();
    }

}
