package djmil.cordacheckers.gameproposal;

import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatingFlow;
import net.corda.v5.application.flows.SubFlow;
import net.corda.v5.application.messaging.FlowMessaging;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

@InitiatingFlow(protocol = "game-proposal")
public class CommitSubFlow implements SubFlow<SecureHash> {

    private final static Logger log = LoggerFactory.getLogger(CommitSubFlow.class);
    private final UtxoSignedTransaction signedTransaction;
    private final MemberX500Name respondenName;

    public CommitSubFlow(UtxoSignedTransaction signedTransaction, MemberX500Name respondenName) {
        this.signedTransaction = signedTransaction;
        this.respondenName = respondenName;
    }

    @CordaInject
    public UtxoLedgerService ledgerService;

    @CordaInject
    public FlowMessaging flowMessaging;

    @Override
    @Suspendable
    public SecureHash call() {
        log.info("GamePropsal commit started");

        final FlowSession session = flowMessaging.initiateFlow(this.respondenName);

        /*
         * Calls the Corda provided finalise() function which gather signatures from the counterparty,
         * notarises the transaction and persists the transaction to each party's vault.
         */

        final List<FlowSession> sessionsList = Arrays.asList(session);

        final SecureHash trxId = ledgerService
            .finalize(this.signedTransaction, sessionsList)
            .getTransaction()
            .getId();

        log.info("GamePropsal commit " +trxId);
        return trxId;
    }
}
