package djmil.cordacheckers.gameproposal;

import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.FlowResult;
import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.ledger.utxo.UtxoLedgerService;

public class ListFlow implements ClientStartableFlow {

    private final static Logger log = LoggerFactory.getLogger(ListFlow.class);

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @CordaInject
    public JsonMarshallingService jsonMarshallingService;

    @Suspendable
    @Override
    public String call(ClientRequestBody requestBody) {
        try {
            log.info("ListFlow.call() called");

            // Queries the VNode's vault for unconsumed states and converts the resulting 
            // List<StateAndRef<GameProposalState>> to a _serializable_ List<ListItem> DTO 
            List<ListItem> unconsumedGameProposaList = utxoLedgerService
                .findUnconsumedStatesByType(GameProposalState.class)
                .stream()
                .map( stateAndRef -> new ListItem(stateAndRef.getState().getContractState()) )
                .collect(Collectors.toList());

            return new FlowResult(unconsumedGameProposaList).toJsonEncodedString(jsonMarshallingService);
        } catch (Exception e) {
            log.warn("ListFlow failed to process utxo request body " + requestBody + " because: " + e.getMessage());
            return new FlowResult(e).toJsonEncodedString(jsonMarshallingService);
        }
    }
    
}
